import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Write a description of class AITank here.
 * 
 * @author (your name) 
 * @version (a version number or a date)
 */
public class AITank  extends Tank
{
    /**
     * the turret associated with this AI tank
     */
    private Turret turret;
    
    private boolean wishesToFire = false;
    private static final int AI_FIRE_DELAY = 100;
    private int fireCounter = 0;
    private static final int AI_WAIT_DIST = 190;
    
    /**
     * constructor - initialise the associated tank turret
     */
    public AITank() 
    {
        turret = new Turret((Vehicle)this);
    }
    
    /**
     * called when the tank is added to the world, adds a turret
     */
    public void addedToWorld(World world) 
    {     
        world.addObject(turret,this.getX(),this.getY());       
    }
    
    /**
     * Act - do whatever the AITank wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {     
        TankWorld curWorld = (TankWorld)this.getWorld();
        if (curWorld.isPlayerAlive()) 
        {
            pointAt(curWorld.getPlayerX(),curWorld.getPlayerY());
            pointTurretAt(curWorld.getPlayerX(),curWorld.getPlayerY());
            engagePlayer();
            processGun();
        } else {
            this.turnTurret(2);
            this.turn(-4);
        }

    }    
    
    /**
     * turn the turret of the tank
     */
    public void turnTurret(int angle) {
        turret.turn(angle);
    }

    /**
     * turn the tank & the turret
     */
    @Override
    public void turn(int angle)
    {
        super.turn(angle);
        turret.turn(angle);
    }
    
    /**
     * point the tank at a specific location
     */
    public void pointAt(int X, int Y) 
    {   
        if (trig.getFinalAngleFrom(this.getX(),this.getY(),X,Y) > this.getRotation()) 
        {
            this.turn(1);
        } else {
            if (trig.getFinalAngleFrom(this.getX(),this.getY(),X,Y) < this.getRotation()) 
            {
                this.turn(-1);
            }
        }
    }
    
    /**
     * go to player & shoot them
     */
    public void engagePlayer() 
    {
        TankWorld curWorld = (TankWorld)this.getWorld();   
        double dist = trig.distFrom(this.getX(),this.getY(),curWorld.getPlayerX(),curWorld.getPlayerY());
        
        if (Math.abs(Math.abs(dist) - AI_WAIT_DIST) > 5) 
        {
            goTo(curWorld.getPlayerX(),curWorld.getPlayerY(),200);            
        }
                
        if (trig.distFrom(this.getX(),this.getY(),curWorld.getPlayerX(),curWorld.getPlayerY()) < Bullet.BULLET_MAX_LIFE*Bullet.BULLET_SPEED) 
        {
            fireGun();  
        }              
    }
    
    /**
     * go to a specified co-ordinate
     */
    public void goTo(int X,int Y,int waitDist)
    {
        if (trig.distFrom(this.getX(),this.getY(),X,Y) > waitDist) {
            this.move(-1);
        } else {
            if (trig.distFrom(this.getX(),this.getY(),X,Y) < waitDist) {
                this.move(1);
            }
        }
    }
    
    /**
     * point the turret at a specified co-ordinate
     */
    public void pointTurretAt(int X, int Y) 
    {
        if (trig.getFinalAngleFrom(this.getX(),this.getY(),X,Y) > turret.getRotation()) 
        {
            this.turnTurret(2);
        } else {
            if (trig.getFinalAngleFrom(this.getX(),this.getY(),X,Y) < turret.getRotation()) 
            {
                this.turnTurret(-2);
            }
        }        
    }
    
    /**
     * ask to fire the gun
     */
    public void fireGun() 
    {
        wishesToFire = true;
    }
    
    /**
     * process the gun
     */
    public void processGun() 
    {
        if (fireCounter >= AI_FIRE_DELAY) 
        {
            if (wishesToFire) 
            {
                wishesToFire = false;
                int[] bulletStart = trig.getOffsetFrom(turret.getX(),turret.getY(),turret.getRotation(),45);
                this.getWorld().addObject(new Bullet(turret.getRotation(),PlayerTank.class,this),bulletStart[0],bulletStart[1]);
            }
            fireCounter = 0;
        }
        fireCounter++;
    }
    
    /**
     * destroy this tank & associated turret
     */
    public void destroy() 
    {
        try 
        {
            this.getWorld().removeObject(turret); //remove the turret associated with me#
            //NOTE: We do not use turret.destroy(); here as that'd destroy this object too.
        } catch (IllegalStateException e) 
        {
            System.out.println("Attempted to remove non existing turret");
        }
        TankWorld curWorld = (TankWorld)this.getWorld(); //find the current tankWorld
        curWorld.removeAITank(); //flag me as removed in the tankWorld
        super.destroy(); //remove me from world
    }   
}
