import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.*;
/**
 * Write a description of class Bullet here.
 * 
 * @author (your name) 
 * @version (a version number or a date)
 */
public class Bullet extends Vehicle
{
    public static final int BULLET_MAX_LIFE = 120;
    public static final int BULLET_SPEED = 2;
    public static final int BULLET_ACCURACY_ERROR = 5;
    public static final int BULLET_REBOUND_CHANCE = 30; 
    
    private boolean requestDestroy = false; //this is set to true when the bullet
    //should be destroyed rather than calling destroy() all over the place.
    
    private int bulletLife; //how long the bullet has lived so far (cycles)
    private Class classToHurt;
    private Vehicle ownerVeh;
    
    public Bullet(int Angle, Class classToHurt, Vehicle ownerVeh) 
    {
        this.classToHurt = classToHurt;
        this.setRotation(Angle);
    }
    
    /**
     * Act - do whatever the Bullet wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {
        testCollisions();
        testBulletLife();
        checkMapEdges();
        processDestroy();
    }    
    
    /**
     * check to see if the bullet has exceeded its maximum lifetime & destroy if so
     */
    public void testBulletLife() {
        if (bulletLife < BULLET_MAX_LIFE) {
            this.move(-BULLET_SPEED);
        } else {
            requestDestroy = true;           
        }
        bulletLife++;   
    }    
    
    /**
     * check to see if the bullet has hit the edge of the world & remove
     */
    public void checkMapEdges() {
        if (this.getWorld() != null) {
            if (this.getX() >= this.getWorld().getWidth() - 1 || this.getX() <= 0) 
            {
                requestDestroy = true;
            }
            if (this.getY() >= this.getWorld().getHeight() - 1 || this.getY() <= 0) 
            {
                requestDestroy = true;
            }        
        }
    }
    
    /**
     * force the bullet to turn around- it has rebounded
     */
    public void rebound() 
    {
        this.turn(180);
    }
    /**
     * check to see if the bullet has hit a vehicle & destroy one if it has
     */
    public void testCollisions() { //if we're intersecting an object and it isn't our owner
        
        if (this.getOneIntersectingObject(classToHurt) != null)
        {
            if (Greenfoot.getRandomNumber(100) > 100 - BULLET_REBOUND_CHANCE) 
            {

                Vehicle vehicleToBreak = (Vehicle)getOneIntersectingObject(classToHurt);
                vehicleToBreak.destroy();
                requestDestroy = true;
            }
            else
            {
                rebound();
            }
        }
    }
    public void processDestroy() 
    {
        if (requestDestroy)
        {
            this.destroy();
        }
    }
    /**
     *  remove the bullet from the world
     */
    public void destroy() 
    {
        //create a crater if we're not intersecting an object
        if (this.getIntersectingObjects(Actor.class).size() == 0 && !this.atWorldEdge()) { 
            TankWorld curWorld = (TankWorld)this.getWorld();
            curWorld.addObject(new SpriteMsg("Crater.png",curWorld.CRATER_LIFE),this.getX(),this.getY()); 
        }           
        //destroy the bullet, garbage collecter will eat it.
        this.getWorld().removeObject(this);
    }
}
